@@ -44,6 +44,9 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
+#ifdef HAVE_XEVIE
+#include <X11/extensions/Xevie.h>
+#endif
 #include "main.h"
 #include "gok.h"
 #include "gok-word-complete.h"
@@ -72,6 +75,8 @@
 
 extern int errno;
 
+static Display *xevie_dpy = NULL;
+
 #define USE_GCONF 1
 
 #define APP_STATIC_BUFF_SZ 30
@@ -170,6 +175,7 @@ static void gok_main_object_state_listener (Accessible* pAccessible);
 static gboolean gok_main_has_xkb_extension ();
 static gboolean gok_main_check_sticky_keys (GtkWidget *widget);
 void gok_main_mouse_button_listener (gint button, gint state, long mods, long timestamp);
+static gboolean gok_main_xevie_io (GIOChannel *source, GIOCondition condition, gpointer data);
 
 /* extern funcs */
 void       gok_input_detach_corepointer (void);
@@ -347,8 +353,9 @@ static int gok_sig_handler(int sig)
 #if ! defined ENABLE_LOGGING_NORMAL
 		if (respawn_on_segv)
 		{
-		    fprintf (stderr, "Restarting GOK.\n"); 
-		    g_spawn_command_line_async ("gok", NULL); 
+		    fprintf (stderr, "Restarting GOK.\n");
+		    g_on_error_stack_trace("gok");
+		    g_spawn_command_line_async ("gok", NULL);
 		}
 #endif
 		break;
@@ -582,7 +589,6 @@ gok_main_open(gint argc, gchar *argv[])
 		LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
 		NULL);
 		
-	
 	if (!bonobo_init (&argc, argv))
 	{
 	    if (gok_args.is_login)
@@ -795,11 +801,24 @@ gok_main_open(gint argc, gchar *argv[])
 	gok_spy_register_windowchangelistener ((void *)gok_main_window_change_listener);
 	gok_spy_register_mousebuttonlistener ((void *)gok_main_mouse_button_listener);	
 	gok_spy_register_objectstatelistener ((void *)gok_main_object_state_listener);
-	
+#ifdef HAVE_XEVIE
+	xevie_dpy = XOpenDisplay(NULL);
+	if (!XevieStart (xevie_dpy)) 
+	  xevie_dpy = NULL;
+	else {
+	  GIOChannel *ioc;
+	  int fd;
+	  XevieSelectInput(xevie_dpy, PointerMotionMask);
+	  gok_log ("xevie opened, PointerMotionMask selected: %d\n", (int)xevie_dpy);
+	  fd = ConnectionNumber(xevie_dpy);
+	  ioc = g_io_channel_unix_new (fd);
+	  g_io_add_watch (ioc, G_IO_IN | G_IO_HUP, gok_main_xevie_io, xevie_dpy);
+	  g_io_channel_unref (ioc);
+	}
+#endif	
 	/* initialize sound */
 	gok_sound_initialize();
 
-
 	/* initialize libusb if it's been selected */
 	window_realize_data->use_libusb = FALSE;
 #ifdef ENABLE_LIBUSB_INPUT
@@ -844,7 +863,7 @@ gok_main_open(gint argc, gchar *argv[])
 	gok_keyboard_initialize();
 	gok_branchbackstack_initialize();
 
-	fprintf (stderr, "login mode = %s\n", gok_args.is_login ? "true" : "false");
+	gok_log ("login mode = %s\n", gok_args.is_login ? "true" : "false");
 	if (!gok_args.is_login) {
 		gok_main_initialize_wordcomplete ();
 		gok_main_initialize_commandprediction ();
@@ -871,8 +890,8 @@ gok_main_open(gint argc, gchar *argv[])
 
 	if (!gok_args.is_login) {
 		gok_main_check_accessibility();
-		if (gok_args.usb_vid_pid == NULL) {
-		        /* we're not using libusb */
+		if (gok_args.usb_vid_pid == NULL && xevie_dpy == NULL) {
+		        /* we're not using libusb or xevie */
 			gok_main_warn_corepointer (FALSE, FALSE, TRUE);
 		}
 	}
@@ -1185,12 +1204,45 @@ gok_main_object_state_listener (Accessible* pAccessible)
 static gboolean gok_main_idle_input (gpointer data)
 {
     input_idle_handler = 0;
+
     if ((gok_motion_event.n_axes > 0) && (gok_motion_event.n_axes < 4)) {
 	gok_scanner_input_motion (gok_motion_event.axes, gok_motion_event.n_axes);
     }
     return FALSE;
 }
 
+static gboolean gok_main_xevie_io (GIOChannel *source,
+				   GIOCondition condition,
+				   gpointer data)
+{
+    XEvent xevie_event;
+
+    while (xevie_dpy && XPending (xevie_dpy)) {
+      XMotionEvent *motion_event;
+      gboolean has_active_key = gok_scanner_key_is_active ();
+      /* Xevie has a motion event for us to check */
+      XNextEvent (xevie_dpy, &xevie_event);
+      switch (xevie_event.type)
+	{
+	case MotionNotify:
+	  motion_event = (XMotionEvent*) &xevie_event;
+	  gok_scanner_mouse_movement (motion_event->x_root, motion_event->y_root);
+	  break;
+	default:
+	  fprintf (stderr, "that's odd, unsolicited xevie event\n");
+	}
+      if (!has_active_key)
+	XevieSendEvent (xevie_dpy, &xevie_event, XEVIE_UNMODIFIED);
+    }
+    return TRUE;
+}
+
+gboolean
+gok_main_attach_new_devices (void)
+{
+  return (xevie_dpy == NULL);
+}
+
 void
 gok_main_queue_input_event (int *motion_data, gint n_axes)
 {
@@ -1274,7 +1326,6 @@ void gok_main_button_listener (gint button, gint state, long mods, long timestam
 			       &win_x, &win_y, &mask_ret);
 		SPI_generateMouseEvent (x, y, ename);
 	}
-
 	if (gok_input_corepointer_detached() || using_libusb == TRUE)
 	{
 		gok_main_mouse_button_listener (button, state, mods, timestamp);
@@ -2767,12 +2818,21 @@ void gok_main_get_geometry (GdkRectangle* pRectangle)
 gboolean
 gok_main_window_contains_pointer (void)
 {
-	gint x, y;
+	gint x=-1, y=-1, w=0, h=0;
+	gint px, py;
 	GdkModifierType mask;
 	GtkWidget *widget = gok_main_get_main_window ();
-	if (gdk_window_get_pointer (widget->window, &x, &y, &mask) != NULL)
-		return TRUE;
-	return FALSE;
+	gok_scanner_get_pointer_location (&px, &py);
+	if (widget->window) {
+	  gdk_window_get_position (widget->window, &x, &y);
+	  x = px-x;
+	  y = py-y;
+	  gdk_window_get_size (widget->window, &w, &h);
+	}
+	if (x < 0 || y < 0 || x > w || y > h)
+	  return FALSE;
+	else
+	  return TRUE;
 }
 
 Display *
